﻿using System;
using System.Windows.Forms;
using HeuristicLab.Collections;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.DebugEngine {

  [View("Operator Trace View")]
  [Content(typeof(ItemList<IOperator>), IsDefaultView = true)]
  public sealed partial class OperatorTraceView : ItemView {

    public new ItemList<IOperator> Content {
      get { return (ItemList<IOperator>)base.Content; }
      set { base.Content = value; }
    }

    public OperatorTraceView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.CollectionReset -= Content_CollectionChanged;
      Content.ItemsAdded -= Content_CollectionChanged;
      Content.ItemsMoved -= Content_CollectionChanged;
      Content.ItemsRemoved -= Content_CollectionChanged;
      Content.ItemsReplaced -= Content_CollectionChanged;
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.CollectionReset += Content_CollectionChanged;
      Content.ItemsAdded += Content_CollectionChanged;
      Content.ItemsMoved += Content_CollectionChanged;
      Content.ItemsRemoved += Content_CollectionChanged;
      Content.ItemsReplaced += Content_CollectionChanged;
    }

    #region Event Handlers (Content)
    void Content_CollectionChanged(object sender, CollectionItemsChangedEventArgs<Collections.IndexedItem<IOperator>> e) {
      if (InvokeRequired)
        Invoke(new EventHandler<CollectionItemsChangedEventArgs<IndexedItem<IOperator>>>(Content_CollectionChanged), sender, e);
      else
        UpdateOperatorTrace();
    }
    #endregion

    private void UpdateOperatorTrace() {
      listView.BeginUpdate();
      listView.Items.Clear();
      foreach (var item in Content) {
        var viewItem = listView.Items.Add(item.Name ?? item.ItemName);
        viewItem.Tag = item;
      }
      listView.EndUpdate();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        listView.Items.Clear();
      } else {
        UpdateOperatorTrace();
      }
    }

    #region Event Handlers (child controls)
    private void listView_ItemActivate(object sender, EventArgs e) {
      if (listView.SelectedItems.Count > 0) {
        IOperator op = listView.SelectedItems[0].Tag as IOperator;
        if (op != null) {
          MainFormManager.MainForm.ShowContent(op);
        }
      }
    }
    #endregion
  }

}
